@@ -6,13 +6,18 @@
 #' @param case Outcome variable. If a variable, this variable is tabulated
 #' against.
 #' @param exposed Exposure variable.
-#' @param bias_parms Numeric vector defining the selection probabilities. This vector has 4 elements between 0 and 1, in the following order:
+#' @param bias_parms Selection probabilities. Either a vector of 4 elements
+#' between 0 and 1 defining the following probabilities in this order can be
+#' provided:
 #' \enumerate{
-#' \item Selection probability among cases exposed,
-#' \item Selection probability among cases unexposed,
-#' \item Selection probability among noncases exposed, and
-#' \item Selection probability among noncases unexposed.
+#' \item Selection probability among cases exposed (1),
+#' \item Selection probability among cases unexposed (2),
+#' \item Selection probability among noncases exposed (3), and
+#' \item Selection probability among noncases unexposed (4).
 #' }
+#' or a single positive selection-bias factor which is the ratio of the exposed
+#' versus unexposed selection probabilities comparing cases and noncases [(1*4)/(2*3)
+#' from above].
 #' @param alpha Significance level.
 #' 
 #' @return A list with elements:
@@ -34,6 +39,10 @@
 #' dimnames = list(c("UM+", "UM-"), c("Mobile+", "Mobile-")),
 #' nrow = 2, byrow = TRUE),
 #' bias_parms = c(.94, .85, .64, .25))
+#' selection(matrix(c(136, 107, 297, 165),
+#' dimnames = list(c("UM+", "UM-"), c("Mobile+", "Mobile-")),
+#' nrow = 2, byrow = TRUE),
+#' bias_parms = 0.43)
 #' @export
 #' @importFrom stats qnorm
 selection <- function(case,
@@ -44,11 +53,13 @@ selection <- function(case,
         bias_parms <- c(1, 1, 1, 1)
     else bias_parms <- bias_parms
     if(!is.vector(bias_parms))
-        stop('The argument bias_parms should be a vector of length 4')
-    if(length(bias_parms) != 4)
-        stop('The argument bias_parms should be made of 4 components in the following order: (1) Selection probability among cases exposed, (2) Selection probability among cases unexposed, (3) Selection probability among noncases exposed, and (4) Selection probability among noncases unexposed')
-    if(!all(bias_parms >= 0 & bias_parms <=1))
-        stop('Selection probabilities should be between 0 and 1')
+        stop('The argument bias_parms should be a vector of length 4.')
+    if(length(bias_parms) != 4 & length(bias_parms) != 1)
+        stop('The argument bias_parms should be made of either a) 4 components in the following order: (1) Selection probability among cases exposed, (2) Selection probability among cases unexposed, (3) Selection probability among noncases exposed, and (4) Selection probability among noncases unexposed; or b) the selection probability.')
+    if(length(bias_parms) == 4 & !all(bias_parms >= 0 & bias_parms <=1))
+        stop('Selection probabilities should be between 0 and 1.')
+    if(length(bias_parms) == 1 & !all(bias_parms >= 0))
+        stop('Selection probability should be positive.')
 
     if(inherits(case, c("table", "matrix")))
         tab <- case
@@ -71,11 +82,18 @@ selection <- function(case,
     lci.or <- exp(log(or) - qnorm(1 - alpha/2) * se.log.or)
     uci.or <- exp(log(or) + qnorm(1 - alpha/2) * se.log.or)
 
-    A0 <- a / bias_parms[1]
-    B0 <- b / bias_parms[2]
-    C0 <- c / bias_parms[3]
-    D0 <- d / bias_parms[4]
-
+    if(length(bias_parms == 4)){
+        A0 <- a / bias_parms[1]
+        B0 <- b / bias_parms[2]
+        C0 <- c / bias_parms[3]
+        D0 <- d / bias_parms[4]
+    } else {
+        A0 <- a / bias_parms
+        B0 <- b / bias_parms
+        C0 <- c / bias_parms
+        D0 <- d / bias_parms
+    }
+    
     tab.corr <- matrix(c(A0, B0, C0, D0), nrow = 2, byrow = TRUE)
     rr.corr <- (A0/(A0 + C0)) / (B0/(B0 + D0))
     or.corr <- (A0/B0) / (C0/D0)
@@ -103,7 +121,11 @@ selection <- function(case,
     rownames(rmatc) <- c("Selection Bias Corrected Relative Risk:",
                          "   Selection Bias Corrected Odds Ratio:")
     colnames(rmatc) <- " "
-    selbias <- (bias_parms[1] * bias_parms[4]) / (bias_parms[2] * bias_parms[3])
+    if(length(bias_parms) == 4){
+        selbias <- (bias_parms[1] * bias_parms[4]) / (bias_parms[2] * bias_parms[3])
+    } else {
+        selbias <- bias_parms
+    }
     res <- list(model = "selection",
                 obs.data = tab,
                 corr.data = tab.corr,
